# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class BlockStorageSummary(resource.Resource):
    base_path = "/volumes/summary"

    # capabilities
    allow_fetch = True

    # Properties
    #: Total size of all the volumes
    total_size = resource.Body("total_size")
    #: Total count of all the volumes
    total_count = resource.Body("total_count")
    #: Metadata of all the volumes
    metadata = resource.Body("metadata")

    _max_microversion = "3.36"
