# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import cloud as openstack

# Initialize and turn on debug logging
openstack.enable_logging(debug=True)

for cloud_name, region_name, image, flavor_id in [
    (
        'my-vexxhost',
        'ca-ymq-1',
        'Ubuntu 16.04.1 LTS [2017-03-03]',
        '5cf64088-893b-46b5-9bb1-ee020277635d',
    ),
    (
        'my-citycloud',
        'Buf1',
        'Ubuntu 16.04 Xenial Xerus',
        '0dab10b5-42a2-438e-be7b-505741a7ffcc',
    ),
    ('my-internap', 'ams01', 'Ubuntu 16.04 LTS (Xenial Xerus)', 'A1.4'),
]:
    # Initialize cloud
    cloud = openstack.connect(cloud=cloud_name, region_name=region_name)

    # Boot a server, wait for it to boot, and then do whatever is needed
    # to get a public ip for it.
    server = cloud.create_server(
        'my-server',
        image=image,
        flavor=dict(id=flavor_id),
        wait=True,
        auto_ip=True,
    )
    # Delete it - this is a demo
    cloud.delete_server(server, wait=True, delete_ips=True)
